package Tests::notify_email;

use Tests::Base;
@ISA = qw( Tests::Base );

use Mail::Sender;

sub execute
{
	my $self = shift;
	my $to = $self -> opt( 'to' );
	my $from = $self -> opt( 'from' ) || $self -> host() -> sentinel() -> mail_from();
	my $subject = $self -> opt( 'subject' ) || 'Notification message';
	my $message = $self -> cdata();
	my @mail_servers;

	my( $mail_sent, $sender, $messages );

	if( ! $to )
	{
		$self -> message( "No 'To:' address passed, cannot send mail" );
		return( 1 );
	}

	if( ! $message )
	{
		$self -> message( "No message passed, why would you do that?" );
		return( 1 );
	}

	if( $self -> opt( 'server' ) )
	{
		push( @mail_servers, split( /\s*,\s*/, $self -> opt( 'server' ) ) );
	}else
	{
		@mail_servers = @{ $self -> host() -> sentinel() -> mail_servers() };
	}

	# Clean some of the space in message, the stream coming from XML::Parser is
	# not very clean
	$message =~ s/^\s+//s;
	$message =~ s/\s+$//s;

	$mail_sent = 0;
	for( @mail_servers )
	{
		do {
			$self -> logmsg( "Trying mailserver: $_" );
			$sender = new Mail::Sender {
				smtp => $_,
				from => $from
			};

			if( $sender -> MailMsg( {
				to => $to,
				subject => sprintf( "(%s): %s",
					$self -> host_addr(),
					$subject ),
				msg => $message
				} ) )
			{
				# OK
				return( 0 );
			}

			$messages .= "$_: $Mail::Sender::Error\n";
		} unless $mail_sent;
	}

	$self -> message( "Could not find a mail host:\n$messages" );
	return( 1 );
}

1;

__END__

=head1 NAME

Tests::notify_email - sends a notification email

=head1 SYNOPSIS

	<notify_email from="from" to="to" subject="subject" server="server">
	Message
	</notify_email>

Requires Mail::Sender perl module

=head1 DESCRIPTION

Used to send out messages, informational or otherwise.  For example, if a process test on
a host fails you may want to try and restart the process.  Using an onfail block
would allow the restart to be attempted, and you could throw in a notify too so you
know it's trying to restart.  e.g.:

	<block name="start-syslogd">
	<exec command="/sbin/syslogd"/>
	<notify_email subject="Warning, trying to restart syslogd">
	You should check why it died, perhaps evil h4x0rs?
	</notify_email>
	</block>

	<host addr="my.box" name="My Box">
	<process proc="syslogd" min="1" max="1" onfail="start-syslogd"/>
	</host>

=head2 from

The email From: address.  If not set, will use the mail_from from the config file.

=head2 to

The to email address, must be supplied or an error will be returned.

=head2 subject

The subject of the email.  If not set will use "Notification message".  This
will always have the current test host prepended, i.e. the default subject becomes:

	(this.host): Notification message

=head2 server

The server to send mail through.  If not set will use the mailers specified (in order)
in the config file.

=head1 SEE ALSO

sentinel, probe, librarian, sentinel.conf

=head1 AUTHOR

Steve Slaven - http://hoopajoo.net
